@@ -68,3 +68,29 @@ ctree(y ~ x1 + x2, data = dat)
 ### one factor with only one level
 dat$x2 <- factor(rep(0, 100))
 try(ctree(y ~ x1 + x2, data = dat))
+
+### weights for sampling without replacement for cforest
+### spotted by Carolin Strobl <carolin.strol@stat.uni-muenchen.de>
+airq <- subset(airquality, !is.na(Ozone))
+cctrl <- cforest_control(replace = FALSE, fraction = 0.5)
+n <- nrow(airq)
+w <- double(n)
+
+### case weights
+x <- runif(w)
+w[x > 0.5] <- 1
+w[x > 0.9] <- 2
+
+rf <- cforest(Ozone ~ .,data = airq, weights = w, control = cctrl)
+rfw <- sapply(rf@ensemble, function(x) x[[2]])
+stopifnot(all(colSums(rfw) == ceiling(sum(w) / 2)))
+stopifnot(max(abs(rfw[w == 0,])) == 0)
+
+### real weights
+w <- runif(n)
+w[1:10] <- 0
+rf <- cforest(Ozone ~ .,data = airq, weights = w, control = cctrl)
+rfw <- sapply(rf@ensemble, function(x) x[[2]])
+stopifnot(all(colSums(rfw) == ceiling(sum(w > 0) / 2)))
+stopifnot(max(abs(rfw[w == 0,])) == 0)
+
